#-------------------------------------------------------------------------------
#   Licensed Materials - Property of IBM
#
#   5724-T01 5724-S98
#
#    Copyright IBM Corp. 2003, 2010. All Rights Reserved.
#
#   US Government Users Restricted Rights - Use, duplication or
#   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#-------------------------------------------------------------------------------
package EngineConfig;


@ISA = qw(Exporter);
@EXPORT = qw(engine_config);

#use diagnostics;
use strict;
use Exporter ();
use AudLog;
use DDM_MANIP_Config;
use Data::Dumper;

use Block_Name_Mapping;
my $block_rename_list = get_block_rename_list();
my $block_manip_list = get_block_manip_list();
#-------------------------------------------------------------------------------
# This perl module contains the configuration information for this parser.
# If you don't understand the format of this configuration file DON'T TOUCH IT.
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# Modify/Add rules here
#-------------------------------------------------------------------------------
my @rules = 
(
     {
         RULE_TYPE => 'SIEMENS_ACL',
         RULE_DESC => 'Process OMC MML create command',
         INPUT_FILE_DESCRIPTION => [ '^.+\.(acl|asc|ACL|ASC)$' ],
         INPUT_DIR_DEPTH        => 5,
         FILENAME_HEADER_FIELDS => {
             ID => '^(.+)\.(acl|asc|ACL|ASC)$',
         },
		 COMMAND_COUNTERS_FOR_HEADER => [],
         COMMAND_CONFIG => 
         {
             BTS => {
                 KEEP_COUNTERS => [ qw(NAME.* BTSM BTS CELLGLID BSIC PLMNP MSTXPMAXGSM CONCELL CELLTYP BCCHFREQ CALLF01 MCC MNC LAC CI ADJ_CI CELLTYPE) ],
				 EXTRACT_COUNTERS => {
                     CELLGLID => {
                         # Extract the MCC, MNC, LAC and CI from the CELLGLID field
                         # Eg. "262"-"77"-6000-630
                         MCC => '^\"(\d+)\"-\"\d+\"-\d+-\d+$',
                         MNC => '^\"\d+\"-\"(\d+)\"-\d+-\d+$',
                         LAC => '^\"\d+\"-\"\d+\"-(\d+)-\d+$',
                         CI  => '^\"\d+\"-\"\d+\"-\d+-(\d+)$',
						 ADJ_CI  => '^\"\d+\"-\"\d+\"-\d+-(\d+)$',
                     },           
                 },
                 NEW_COUNTERS => {
                     BTSM => [ "NAMEBTSM" ], 
                     BTS  => [ "NAMEBTS" ],
					 CELLTYPE => [ "CELLTYP" ]
                 },
                 OUTPUT_BLOCK_NAME => 'CREATEBTS',                
             },

             TRX => {
                 KEEP_COUNTERS => [ qw(NAME.* BTSM BTS TRX TRXFREQ) ],               
                 NEW_COUNTERS => {
                     BTSM => [ "NAMEBTSM" ], 
                     BTS  => [ "NAMEBTS" ],
                     TRX  => [ "NAMETRX" ],
                 },
                 OUTPUT_BLOCK_NAME => 'CREATETRX',                
             },

             CHAN => {
                 KEEP_COUNTERS => [ qw(NAME.* BTSM BTS TRX CHAN CHTYPE) ],               
                 NEW_COUNTERS => {
                     BTSM => [ "NAMEBTSM" ], 
                     BTS  => [ "NAMEBTS" ],
                     TRX  => [ "NAMETRX" ],
                     CHAN => [ "NAMECHAN" ],
                 },
                 OUTPUT_BLOCK_NAME => 'CREATECHAN',                
             },

             FRL => {
                 KEEP_COUNTERS => [ qw(NAME.* FRL PCUID GTS) ],
                 EXTRACT_COUNTERS => {
                     PCUID => {
                         # Extract the PCUID from the PCUID field
                         # Eg. PCU:0
						 #PCUID => '^\w+:(\d+)$', Siemens BR9
                         PCUID => '^\w*:*(\d+)$',
                     },
                 },
                 NEW_COUNTERS => {
                     FRL    => [ "NAMEFRL" ],                                                 
                 },
                 OUTPUT_BLOCK_NAME => 'CREATEFRL',                
             },

             NSVC => {
                 KEEP_COUNTERS => [ qw(NAME.* NSVC NSVCI NSVLI PCU) ],               
                 NEW_COUNTERS => {
                     NSVC => [ "NAMENSVC" ],
					 PCU => [ "NAMENSVC" ],
                 },
                 OUTPUT_BLOCK_NAME => 'CREATENSVC',                
             },
             PCU => {
                 KEEP_COUNTERS => [ qw(NAME.* PCU NMO NSEI) ],               
                 NEW_COUNTERS => {
                     PCU => [ "NAMEPCU" ],  
                 },
                 OUTPUT_BLOCK_NAME => 'CREATEPCU',                
             },

             NSE => {
                 KEEP_COUNTERS => [ qw(NAME.* NSE NMO NSEI) ],               
                 NEW_COUNTERS => {
                     NSE => [ "NAMENSE" ],  
                 },
                 OUTPUT_BLOCK_NAME => 'CREATENSE',                
             },

             PTPPKF => {            
                 KEEP_COUNTERS => [ qw(NAME.* BTSM BTS PTPPKF RACODE RACOL CPCUN PCUN) ], 
				 NEW_COUNTERS => {
                     BTSM   => [ "NAMEBTSM" ],  
                     BTS    => [ "NAMEBTS" ],
                     PTPPKF => [ "NAMEPTPPKF" ],
					 PCUN => [ "CPCUN" ],
                 },
                 OUTPUT_BLOCK_NAME => 'CREATEPTPPKF',                
             },
			 
			 TGTBTS => {            
                 KEEP_COUNTERS => [ qw(NAME.* TGTBTS BCCHFREQ BSIC CBQ CELLBARR CELLGLID MSTXPMAXCH MCC MNC LAC CI ADJ_CI) ], 
				 EXTRACT_COUNTERS => {
                     CELLGLID => {
                         # Extract the MCC, MNC, LAC and CI from the CELLGLID field
                         # Eg. "262"-"77"-6000-630
                         MCC => '^\"(\d+)\"-\"\d+\"-\d+-\d+$',
                         MNC => '^\"\d+\"-\"(\d+)\"-\d+-\d+$',
                         LAC => '^\"\d+\"-\"\d+\"-(\d+)-\d+$',
                         CI  => '^\"\d+\"-\"\d+\"-\d+-(\d+)$',
						 ADJ_CI  => '^\"\d+\"-\"\d+\"-\d+-(\d+)$',
                     },           
                 },
				 NEW_COUNTERS => {
                     TGTBTS   => [ "NAMETGTBTS" ],  
                 },
                 OUTPUT_BLOCK_NAME => 'CREATETGTBTS',                
             },
         },
         
     },
	 
	 {  
        RULE_TYPE                        => 'XML_3GPP',
        RULE_DESC                        => '3GPP XML Interface - Siemens BSS',
        XML_3GPP_HANDLER_PACKAGE_NAME    => 'XML_3GPP_Handler',
        INPUT_FILE_DESCRIPTION           => ['\w{1}\d{8}\.\d{4}[+-]?\d{0,4}-\d{8}\.\d{4}[+-]?\d{0,4}_\w+'],
        INPUT_DIR_DEPTH                  => 5,
        ORDER_OF_FILES                   => 'OLDEST_FIRST',
        DIRECTORY_HEADER_FIELDS     => {
            NETWORK_ID  => '\/NETWORK_?([^\/]+)\/?',
            REGION_ID => '\/[^\/]+\/REGION_?([^\/]+)\/?',
        },
        FILENAME_HEADER_FIELDS           => {
	      FILENAME => '^(.*\w{1}\d{8}\.\d{4}[+-]?\d{0,4}-\d{8}\.\d{4}[+-]?\d{0,4}_.+)$',
              Type       => '^.*(A|B|C|D).+$',
			  StartDate  => '^.*\w{1}(\d{8})\.\d{4}[+-]?\d{0,4}-\d{8}\.\d{4}[+-]?\d{0,4}_.+$',
			  Start => '^.*(\w{1}\d{8}\.\d{4})[+-]?\d{0,4}-\d{8}\.\d{4}[+-]?\d{0,4}_.+$',
			  End => '^.*\w{1}\d{8}\.\d{4}[+-]?\d{0,4}-(\d{8}\.\d{4})[+-]?\d{0,4}_.+$',
			  End2 => '^.*\w{1}\d{8}\.\d{4}[+-]?\d{0,4}-\d{8}\.\d{4}[+-]?\d{0,4}_(.+)$',
        },
		XML_PIF_HEADER_ELEMENT_NAMES     => ['managedElement'],
		XML_UNIQUE_PIF_BLOCK_HEADER_ELEMENT_NAMES    => ['granPeriod'],

		XML_UNIQUE_PIF_BLOCK_ELEMENT_NAME    => 'measInfo',		
        XML_3GPP_FILENAME_VALIDATION     =>    {
                                                Type      => '^.*(A|B|C|D).+$',
                                                StartDate => '^.*\w{1}(\d{8}).+$',
                                                StartTime => '^.*\w{1}\d{8}\.(\d{4}).+$'
                                            },
        XML_HEADER_INFO_FOR_PIF_FILENAME     => ['managedElement_ManagedElement_BSS_ID','granPeriod_endTime_Date','granPeriod_endTime_Time','End','End2'],
		
		#Audit
		XML_OUTPUT_BLOCK_ELEMENT_NAME        => '',
		#XML_OUTPUT_BLOCK_DESCRIPTION         => '^(\w+:\d+) \w+:\d+',
		XML_MEASUREMENT_TYPE_ELEMENT_SEQ     => 'measTypes', 
        XML_MEASUREMENT_VALUE_ELEMENT_NAME   => 'measValue',
        XML_MEASUREMENT_RESULTS_ELEMENT_SEQ  => 'measResults',	
        XML_ENABLE_PIF_PER_MOID              => '0',
        XML_SUSPECT_FLAG_ELEMENT_NAME        => 'suspect',
		XML_LIST_DELIMITER                   => " ",
        XML_MAX_COUNTER_NAME_SIZE            => 1000,
        XML_ENABLE_PIF_COUNTER_IN_FILENAME   => 1,
        XML_DDM_MANIP_RULES                  => ddm_manip_config(),
		XML_PARSE_ELEMENT_NAME_ATTR          => ['measValue'],

		HEADER_DATA_RECORD_PROCESSING => sub {
		my ($blkname_ref, $h_ref, $d_ref) = @_;

		my $hashstring = "";
		my @deletekeys = ();
		
		foreach my $key (keys %{$d_ref})
		{
			if($key =~ /\./)
			{
				my $newkey = $key;
				$newkey =~ s/\./_/g;
				$$d_ref{$newkey} = $$d_ref{$key};
				delete $$d_ref{$key};
			}
		}
		
		$$h_ref{granPeriod_duration} =~ /PT(\d+)M/;
		$$h_ref{GRANULARITY} = $1;
		delete $$h_ref{granPeriod_duration};
		$$h_ref{ENDDATE} = $$h_ref{granPeriod_endTime_Date};
		delete $$h_ref{granPeriod_endTime_Date};
		$$h_ref{ENDTIME} = $$h_ref{granPeriod_endTime_Time};
		delete $$h_ref{granPeriod_endTime_Time};
		
		#Choose block name
		$$d_ref{measObjLdn} =~ s/\"//g;
		if ($$d_ref{measObjLdn} =~ /^(.*)\s+\-\s+(.*)$/) {
			my $SCANBTSIHO = 'FALSE';
			my @arr1;
			my @arr2;
			foreach my $counter (sort (keys %{$d_ref}))
			{
				if($counter =~ /(AININIRH|SININIRH)/)
				{	
					$SCANBTSIHO = 'TRUE';
					last;
				}
			}
			
			if ($SCANBTSIHO eq 'TRUE')
			{		
				@arr1 = split(",", $2);
				@arr2 = split(",", $1);
			}
			else
			{		
				@arr1 = split(",", $1);
				@arr2 = split(",", $2);
			}
			
			@arr1 = sort(@arr1);
			foreach my $cvar (@arr1)
			{
				my @valpair = split(":", $cvar);
				$hashstring .= $valpair[0]."_";
				$$d_ref{$valpair[0]."_ID"} = $valpair[1];
			}
				
			@arr2 = sort(@arr2);
			foreach my $cvar (@arr2)
			{
				my @valpair = split(":", $cvar);
				$hashstring .= $valpair[0]."_";
				if($valpair[0] =~ /^TGTFDD$/) {
					$valpair[0] = 'ADJ_FDD_NO';
					$$d_ref{$valpair[0]} = $valpair[1];
				} elsif ($valpair[0] =~ /^TGTBTS$/) {
					$valpair[0] = 'ADJ_TGTBTS_NO';
					$$d_ref{$valpair[0]} = $valpair[1];
				} else {
					$$d_ref{"ADJ_".$valpair[0]."_ID"} = $valpair[1];
				}
			}
			
			$$d_ref{measObjLdn} =~ s/\s+\-\s+/_/; 	#Fixpack fix bad lif change - to _
			
		} else {
			my @arr = split(",", $$d_ref{measObjLdn});
			@arr = sort(@arr);
			foreach my $cvar (@arr)
			{
				my @valpair = split(":", $cvar);
				$hashstring .= $valpair[0]."_";
				if($valpair[0] =~ /^SS7L$/) 
				{
					$valpair[0] = 'LINKSET';
					$$d_ref{$valpair[0]} = $valpair[1];
				}
				elsif($valpair[0] =~ /^CHAN$/) 
				{
					$valpair[0] = 'CHANNEL_ID';
					$$d_ref{$valpair[0]} = $valpair[1];
				}
				elsif($valpair[0] =~ /^ATMVP$/)
				{
					$valpair[0] = 'ATMVPC_ID';
					$$d_ref{$valpair[0]} = $valpair[1];
				}
				elsif($valpair[0] =~ /^ATMVC$/)
				{
					$valpair[0] = 'ATMVCC_ID';
					$$d_ref{$valpair[0]} = $valpair[1];
				}
				else 
				{
					$$d_ref{$valpair[0]."_ID"} = $valpair[1];
				}
			}
		}
		
		$$d_ref{BSC_ID} = $$d_ref{BSS_ID};
		if(exists $$d_ref{ADJ_BSS_ID})
		{	
		    $$d_ref{ADJ_BSC_ID} = $$d_ref{ADJ_BSS_ID};
		    $$d_ref{ADJ_BSS_ID}='';
		}
		$hashstring =~ s/_$//;
		
		my $new_block_name;
        if (exists ($block_rename_list->{$hashstring})) 
		{	$new_block_name = $block_rename_list->{$hashstring};	}
		else
		{	
			$new_block_name = 'NOBLOCK';
		}

		if ($block_manip_list->{$new_block_name} =~ /SPECIAL/)
		{
			for ($block_manip_list->{$new_block_name}) {
				/^SPECIAL_SCANBTS$/ && do {
					my @keys = sort (keys %{$d_ref});
					for (my $i=0; $i<=($#keys + 1); $i++)
					{
						if(not ($keys[$i] =~ /(_ID$|^ADJ_|^CI$|^CELLTYPE$|^CONCELL$|^PCUN$|^ENDDATE$|^ENDTIME$|^GRANULARITY$)/))
						{
							if($keys[$i] =~ /(AMRCHDIS|AMRFER|AMRFRMDL|AMRFRMUL|AMRRXQUAL|AMRSIDUL|AMRVAR|FACCHSUP|MBTSPWR|NCRLBRQU|NDEFCCCH|NFRMDLARP|NFRMULARP|NINVRACH|NTDMAGCH|NTDMPCH|RXQUARP|SACCHSUP)/)
							{	
								$new_block_name = 'SCANBTSE';
								last;
							}
							elsif($keys[$i] =~ /(NFRMULARP)/)
							{	
								$new_block_name = 'SCANBTSE';
								last;
							}
							elsif($keys[$i] =~ /(^ATCHSMBS_[1-2]$|^MEBUSTCH_[1-2]$|^NAVTCH_[1-6]$|^NRDEFTCH_[1-6]$|^NRFLTCH_[1-9]$|^NRFLTCH_1[0-8]$|^TNTCHCL_[1-2]$)/)
							{	
								$new_block_name = 'SCANBTS';
								last;
							}
							elsif($keys[$i] =~ /(NRCLRCMD|NRCLRREQ|NSUCCHPC)/)
							{	
								$new_block_name = 'SCANBTS';
								last;
							}
							elsif($keys[$i] =~ /(^MEBUTSLY_109$|^MEBUTSLY_1[1-7][0-9]$|^MEBUTSLY_180$)/)
							{
								$new_block_name = 'SCANBTS_STANDARD';
								last;	
							}
							elsif($keys[$i] =~ /(^ATCHSMBS_[5-8]$|^MEBUSTCH_[5-6]$|^NAVTCH_[7-9]$|^NAVTCH_1[0-8]$|^NRDEFTCH_[7-9]$|^NRDEFTCH_1[0-8]$|^NRFLTCH_19$|^NRFLTCH_[2-4][0-9]$|^NRFLTCH_5[0-4]$|^TNTCHCL_[3-6]$)/)
							{	
								$new_block_name = 'TEMP_BTS'; #It is a SCANBTS_CONCENTRIC
								last;
							}
							elsif($keys[$i] =~ /(^ATCHSMBS_9$|^ATCHSMBS_1[0-2]$|^MEBUSTCH_[7-9]$|^MEBUSTCH_10$|^NAVTCH_19$|^NAVTCH_2[0-9]$|^NAVTCH_30$|^NRDEFTCH_19$|^NRDEFTCH_2[0-9]$|^NRDEFTCH_30$|^NRFLTCH_5[6-9]$|^NRFLTCH_[6-8][0-9]$|^NRFLTCH_90$|^TNTCHCL_[7-9]$|^TNTCHCL_10$)/)
							{	
								$new_block_name = 'TEMP_BTS'; #It is a SCANBTS_EXTENDED
								last;
							}
							elsif($keys[$i] =~ /(^MEBUTSLY_3[7-9]$|^MEBUTSLY_[4-9][0-9]$|^MEBUTSLY_10[0-8]$)/)
							{
								$new_block_name = 'TEMP_BTS'; #It maybe a SCANBTS_CONCENTRIC or SCANBTS_EXTENDED
								last;	
							}
							elsif($i ==($#keys + 1))
							{	$new_block_name = 'SCANBTS';	}
						}
					}
					#for BTS_AGG_TO_BSC purpose
					if($new_block_name eq 'SCANBTS')
					{	$d_ref->{block} = 'SCANBTS';	}
					last;
				};
				/^SPECIAL_SCANBTSM$/ && do {
					my $found = 'FALSE';
					foreach my $counter (keys %{$d_ref})
					{
						if($counter =~ /(ABISPDIS|ABISPSUP|IFRMABSC)/)
						{	
							$new_block_name = 'SCANFBTSM';
							$found = 'TRUE';
							last;
						}
					}
					if ($found eq 'FALSE')
					{	$new_block_name = 'SCANBTSM';	}
					last;
				};
				/^SPECIAL_SCANBSC$/ && do {
					my %sorted1;
					my %sorted2;
					my @similar1;
					my @similar2;
					
					foreach my $element (keys %{$d_ref})
					{
						$element =~ /(\w+_\d+)/;
						my $name = $1;
						$element =~ /\w+_(\d+)/;
						my $num = $1;
						if($element =~ /BSCPRCLD/)
						{	
							$sorted1{$num} = $name;
						}
						if($element =~ /CNSPRCLD/)
						{	
							$sorted2{$num} = $name;
						}
					}
					
					foreach my $element (sort { $a <=> $b }(keys (%sorted1)))
					{	push @similar1, $sorted1{$element};	}
					foreach my $element (sort { $a <=> $b }(keys (%sorted2)))
					{	push @similar2, $sorted2{$element};	}
					
					$similar1[0] =~ /BSCPRCLD_(\d+)/;
					my $minnum1 = $1;
					$similar1[$#similar1] =~ /BSCPRCLD_(\d+)/;
					my $maxnum1 = $1;
					
					$similar2[0] =~ /CNSPRCLD_(\d+)/;
					my $minnum2 = $1;
					$similar2[$#similar2] =~ /CNSPRCLD_(\d+)/;
					my $maxnum2 = $1;
					
					if(($minnum2 = 2) && ($maxnum2 = 3) && (($#similar2+1) == 2))
					{	
						$new_block_name='TEMP_EBSC'; #it is a SCANBSC_EBSC
					}
					elsif(($minnum2 = 1) && ($maxnum2 = 3) && (($#similar2+1) == 3))
					{	$new_block_name='SCANBSC';	}
					elsif(($minnum1 >= 8) && ($maxnum1 <= 98) && (($#similar1+1) == 74))
					{	$new_block_name='TEMP_EBSC'; } #it is a SCANBSC_EBSC_BASIC
					elsif(($minnum1 >= 8) && ($maxnum1 <= 98) && (($#similar1+1) == 78))
					{	$new_block_name='TEMP_EBSC'; } #it is a SCANBSC_EBSC_EXTENDED
					else
					{	$new_block_name='SCANBSC';	}
					last;
				};
				/SPECIAL_SCANBTSHO$/ && do {
					foreach my $counter (keys %{$d_ref})
					{
						if($counter =~ /(AININIRH|SININIRH)/)
						{	
							$new_block_name = 'SCANBTSIHO';
							last;
						} 
						elsif($counter =~ /(AOUINIRH|SOUINIRH|UNINHOIE)/)
						{
							$new_block_name = 'SCANBTSOHOI';
							last;
						}
					}
					last;
				};
			}
		}
        $$blkname_ref = $new_block_name;

        return 0;
		}
	}
); 

sub engine_config {
    return \@rules;
}

1;
